/*
 * Iris is a World Generator for Minecraft Bukkit Servers
 * Copyright (c) 2021 Arcane Arts (Volmit Software)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package com.volmit.iris.util.noise;

import com.volmit.iris.util.function.NoiseProvider;
import lombok.Builder;
import lombok.Data;

@Builder
@Data
public class WormIterator2 {
    private transient Worm2 worm;
    private transient NoiseProvider noise;
    private int x;
    private int z;
    private int maxDistance;
    private int maxIterations;

    public boolean hasNext() {
        double dist = maxDistance - (Math.max(Math.abs(worm.getX().getVelocity()), Math.abs(worm.getZ().getVelocity())) + 1);
        return maxIterations > 0 &&
                ((x * x) - (worm.getX().getPosition() * worm.getX().getPosition()))
                        + ((z * z) - (worm.getZ().getPosition() * worm.getZ().getPosition())) < dist * dist;
    }

    public Worm2 next() {
        if (worm == null) {
            worm = new Worm2(x, z, 0, 0);
            return worm;
        }

        worm.getX().setVelocity(noise.noise(worm.getX().getPosition(), 0));
        worm.getZ().setVelocity(noise.noise(worm.getZ().getPosition(), 0));
        worm.step();

        return worm;
    }
}
